@@ -34,7 +34,10 @@ import wrapt
 
 from copy import copy as shallow_copy
 from contextlib import contextmanager
-from functools import wraps
+from functools import (
+    lru_cache,
+    wraps,
+)
 from time import sleep
 import inspect
 from itertools import tee
@@ -77,6 +80,8 @@ on_msys_tainted_paths = on_windows \
                         and os.environ.get('MSYSTEM', '')[:4] in ('MSYS', 'MING')
 
 
+# Takes ~200msec, so should not be called at import time
+@lru_cache()  # output should not change through life time of datalad process
 def get_linux_distribution():
     """Compatibility wrapper for {platform,distro}.linux_distribution().
     """
@@ -91,16 +96,12 @@ def get_linux_distribution():
     return result
 
 
-try:
-    linux_distribution_name, linux_distribution_release \
-        = get_linux_distribution()[:2]
-    on_debian_wheezy = on_linux \
-                       and linux_distribution_name == 'debian' \
-                       and linux_distribution_release.startswith('7.')
-except:  # pragma: no cover
-    # MIH: IndexError?
-    on_debian_wheezy = False
-    linux_distribution_name = linux_distribution_release = None
+# TODO: deprecated, remove in 0.15
+# Wheezy EOLed 2 years ago, no new datalad builds from neurodebian
+on_debian_wheezy = False
+# Those weren't used for any critical decision making, thus we just set them to None
+# Use get_linux_distribution() directly where needed
+linux_distribution_name = linux_distribution_release = None
 
 # Maximal length of cmdline string
 # Query the system and use hardcoded "knowledge" if None
@@ -1067,7 +1068,7 @@ def get_tempfile_kwargs(tkwargs=None, prefix="", wrapped=None):
             ([prefix] if prefix else []) +
             ([''] if (on_windows or not wrapped) else [wrapped.__name__]))
 
-    directory = os.environ.get('DATALAD_TESTS_TEMP_DIR')
+    directory = os.environ.get('TMPDIR')
     if directory and 'dir' not in tkwargs_:
         tkwargs_['dir'] = directory
 
@@ -1082,13 +1083,13 @@ def line_profile(func):
     prof = line_profiler.LineProfiler()
 
     @wraps(func)
-    def newfunc(*args, **kwargs):
+    def  _wrap_line_profile(*args, **kwargs):
         try:
             pfunc = prof(func)
             return pfunc(*args, **kwargs)
         finally:
             prof.print_stats()
-    return newfunc
+    return  _wrap_line_profile
 
 
 @optional_args
@@ -1118,7 +1119,7 @@ def collect_method_callstats(func):
     toppath = op.dirname(__file__) + op.sep
 
     @wraps(func)
-    def newfunc(*args, **kwargs):
+    def  _wrap_collect_method_callstats(*args, **kwargs):
         try:
             self = args[0]
             stack = traceback.extract_stack()
@@ -1155,7 +1156,7 @@ def collect_method_callstats(func):
     import atexit
     atexit.register(print_stats)
 
-    return newfunc
+    return  _wrap_collect_method_callstats
 
 
 # Borrowed from duecredit to wrap duecredit-handling to guarantee failsafe
